@@ -57,13 +57,14 @@ static void  gdl_dock_bar_destroy         (GtkObject       *object);
 
 static void  gdl_dock_bar_attach          (GdlDockBar      *dockbar,
                                            GdlDockMaster   *master);
+static void gdl_dock_bar_remove_item      (GdlDockBar      *dockbar,
+                                           GdlDockItem     *item);
 
 /* ----- Class variables and definitions ----- */
 
 struct _GdlDockBarPrivate {
     GdlDockMaster   *master;
     GSList          *items;
-    GtkTooltips     *tooltips;
     GtkOrientation   orientation;
     GdlDockBarStyle  dockbar_style;
 };
@@ -128,11 +129,8 @@ gdl_dock_bar_instance_init (GdlDockBar *dockbar)
     dockbar->_priv = g_new0 (GdlDockBarPrivate, 1);
     dockbar->_priv->master = NULL;
     dockbar->_priv->items = NULL;
-    dockbar->_priv->tooltips = gtk_tooltips_new ();
     dockbar->_priv->orientation = GTK_ORIENTATION_VERTICAL;
     dockbar->_priv->dockbar_style = GDL_DOCK_BAR_BOTH;
-    g_object_ref (dockbar->_priv->tooltips);
-    gtk_object_sink (GTK_OBJECT (dockbar->_priv->tooltips));
 }
 
 static void
@@ -176,6 +174,13 @@ gdl_dock_bar_set_property (GObject         *object,
     };
 }
 
+static void
+on_dock_item_foreach_disconnect (GdlDockItem *item, GdlDockBar *dock_bar)
+{
+    g_signal_handlers_disconnect_by_func (item, gdl_dock_bar_remove_item,
+                                          dock_bar);
+}
+
 static void
 gdl_dock_bar_destroy (GtkObject *object)
 {
@@ -183,6 +188,13 @@ gdl_dock_bar_destroy (GtkObject *object)
 
     if (dockbar->_priv) {
         GdlDockBarPrivate *priv = dockbar->_priv;
+
+        if (priv->items) {
+            g_slist_foreach (priv->items,
+                             (GFunc) on_dock_item_foreach_disconnect,
+                             object);
+            g_slist_free (priv->items);
+        }
         
         if (priv->master) {
             g_signal_handlers_disconnect_matched (priv->master,
@@ -192,11 +204,6 @@ gdl_dock_bar_destroy (GtkObject *object)
             priv->master = NULL;
         }
 
-        if (priv->tooltips) {
-            g_object_unref (priv->tooltips);
-            priv->tooltips = NULL;
-        }
-        
         dockbar->_priv = NULL;
 
         g_free (priv);
@@ -228,6 +235,9 @@ gdl_dock_bar_remove_item (GdlDockBar  *dockbar,
     g_assert (button != NULL);
     gtk_container_remove (GTK_CONTAINER (dockbar), button);
     g_object_set_data (G_OBJECT (item), "GdlDockBarButton", NULL);
+    g_signal_handlers_disconnect_by_func (item,
+                                          G_CALLBACK (gdl_dock_bar_remove_item),
+                                          dockbar);
 }
 
 static void
@@ -311,7 +321,7 @@ gdl_dock_bar_add_item (GdlDockBar  *dockbar,
     gtk_container_add (GTK_CONTAINER (button), box);
     gtk_box_pack_start (GTK_BOX (dockbar), button, FALSE, FALSE, 0);
 
-    gtk_tooltips_set_tip (priv->tooltips, button, name, name);
+    gtk_widget_set_tooltip_text (button, name);
     g_free (name);
 
     g_object_set_data (G_OBJECT (item), "GdlDockBar", dockbar);
@@ -320,6 +330,11 @@ gdl_dock_bar_add_item (GdlDockBar  *dockbar,
                       G_CALLBACK (gdl_dock_bar_item_clicked), item);
 
     gtk_widget_show_all (button);
+    
+    /* Set up destroy notify */
+    g_signal_connect_swapped (item, "destroy",
+                              G_CALLBACK (gdl_dock_bar_remove_item),
+                              dockbar);
 }
 
 static void
